#
# This file is part of PCAP BGP Parser (pbgpp)
#
# Copyright 2016-2017 DE-CIX Management GmbH
# Author: Tobias Hannaske <tobias.hannaske@de-cix.net>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

from pbgpp.BGP.Statics import BGPStatics


class BGPUpdateFlags:
    def __init__(self, payload):
        # Using bitmask to parse bit flags
        # First 4 bit of flag-byte are in use - the following 4 bit need to be ignored
        self.optional = True if (payload & BGPStatics.UPDATE_FLAG_OPTIONAL) is not 0 else False
        self.transitive = True if (payload & BGPStatics.UPDATE_FLAG_TRANSITIVE) is not 0 else False
        self.partial = True if (payload & BGPStatics.UPDATE_FLAG_PARTIAL) is not 0 else False
        self.length = True if (payload & BGPStatics.UPDATE_FLAG_LENGTH) is not 0 else False
